@@ -7,7 +7,7 @@ use color_eyre::eyre::{Context, Result};
 use displaydoc::Display;
 use git2::Repository;
 use log::{debug, error, info, trace};
-use rayon::prelude::*;
+use rayon::{iter::Either, prelude::*};
 use thiserror::Error;
 use walkdir::WalkDir;
 
@@ -17,36 +17,49 @@ use crate::{
     opts::GenerateGitConfig,
     tasks::{
         git::{GitConfig, GitRemote},
-        task::Task,
-        ResolveEnv,
+        task::{Task, TaskStatus},
+        ResolveEnv, TaskError,
     },
 };
 
-pub fn run(generate_git_configs: &[GenerateGitConfig]) -> Result<()> {
-    let errors: Vec<_> = generate_git_configs
-        .par_iter()
-        .map(|config| run_single(config))
-        .filter_map(Result::err)
-        .collect();
+pub fn run(configs: &[GenerateGitConfig]) -> Result<TaskStatus> {
+    let (statuses, errors): (Vec<_>, Vec<_>) =
+        configs
+            .par_iter()
+            .map(run_single)
+            .partition_map(|x| match x {
+                Ok(status) => Either::Left(status),
+                Err(e) => Either::Right(e),
+            });
+
     if errors.is_empty() {
-        Ok(())
+        if statuses.iter().all(|s| matches!(s, TaskStatus::Skipped)) {
+            Ok(TaskStatus::Skipped)
+        } else {
+            Ok(TaskStatus::Passed)
+        }
     } else {
         for error in &errors {
-            error!("{:?}", error);
+            error!("{error:?}");
         }
         let first_error = errors.into_iter().next().ok_or(E::UnexpectedNone)?;
         Err(first_error)
     }
 }
 
-pub fn run_single(generate_git_config: &GenerateGitConfig) -> Result<()> {
+pub fn run_single(generate_git_config: &GenerateGitConfig) -> Result<TaskStatus> {
     debug!(
         "Generating git config for: {path}",
         path = generate_git_config.path.display()
     );
     let mut git_task = Task::from(&generate_git_config.path)?;
-    debug!("Existing git config: {:?}", git_task);
+    debug!("Existing git config: {git_task:?}");
+    let name = git_task.name.as_str();
     let mut git_configs = Vec::new();
+    let home_dir = dirs::home_dir().ok_or(E::MissingHomeDir)?;
+    let home_dir = home_dir.to_str().ok_or_else(|| E::InvalidUTF8Path {
+        path: home_dir.clone(),
+    })?;
     for path in find_repos(
         &generate_git_config.search_paths,
         generate_git_config.excludes.as_ref(),
@@ -55,33 +68,35 @@ pub fn run_single(generate_git_config: &GenerateGitConfig) -> Result<()> {
             &path,
             generate_git_config.prune,
             &generate_git_config.remote_order,
+            home_dir,
         )?);
     }
-    // TODO(gib): keep old branch names.
+
     git_configs.sort_unstable_by(|c1, c2| c1.path.cmp(&c2.path));
-    let toml_configs = git_configs
-        .into_iter()
-        .map(toml::Value::try_from)
-        .collect::<Result<Vec<_>, _>>()?;
-    git_task.config.data = Some(toml_configs.into());
-    debug!("New git config: {:?}", git_task);
+
+    git_task.config.data = Some(serde_yaml::to_value(git_configs)?);
+
+    debug!("New git config: {git_task:?}");
     let mut serialized_task = GENERATED_PRELUDE_COMMENT.to_owned();
-    serialized_task.push_str(&toml::to_string_pretty(&git_task.config)?);
-    trace!("New toml file: <<<{}>>>", serialized_task);
+    serialized_task.push_str(&serde_yaml::to_string(&git_task.config)?);
+    trace!("New yaml file: <<<{serialized_task}>>>");
+    if serialized_task == fs::read_to_string(&generate_git_config.path)? {
+        info!("Skipped task '{name}' as git repo layout unchanged.",);
+        return Ok(TaskStatus::Skipped);
+    }
+
     fs::write(&generate_git_config.path, serialized_task)?;
     info!(
-        "Git repo layout generated for task '{}' and written to '{:?}'",
-        git_task.name, generate_git_config.path
+        "Git repo layout generated for task '{name}' and written to '{path:?}'",
+        path = generate_git_config.path
     );
-    Ok(())
+    Ok(TaskStatus::Passed)
 }
 
-// False-positives on Vec::new(), see https://github.com/rust-lang/rust-clippy/issues/3410
-#[allow(clippy::use_self)]
 impl ResolveEnv for Vec<GenerateGitConfig> {
-    fn resolve_env<F>(&mut self, env_fn: F) -> Result<()>
+    fn resolve_env<F>(&mut self, env_fn: F) -> Result<(), TaskError>
     where
-        F: Fn(&str) -> Result<String>,
+        F: Fn(&str) -> Result<String, TaskError>,
     {
         for config in self.iter_mut() {
             config.path = PathBuf::from(env_fn(&config.path.to_string_lossy())?);
@@ -107,36 +122,49 @@ impl ResolveEnv for Vec<GenerateGitConfig> {
 fn find_repos(search_paths: &[PathBuf], excludes: Option<&Vec<String>>) -> Vec<PathBuf> {
     let mut repo_paths = Vec::new();
     for path in search_paths {
-        trace!("Searching in '{}'", &path.display());
-        for entry in WalkDir::new(path)
-            .into_iter()
-            .filter_entry(|e| {
-                if let Some(ex) = excludes {
-                    let s = e.path().to_str().unwrap_or("");
-                    for exclude in ex {
-                        if s.contains(exclude) {
-                            return false;
-                        }
+        trace!("Searching in '{path:?}'");
+
+        let mut it = WalkDir::new(path).into_iter();
+        'walkdir: loop {
+            let entry = match it.next() {
+                None => break,
+                Some(Err(_)) => continue,
+                Some(Ok(entry)) => entry,
+            };
+
+            // Exclude anything from the excludes list.
+            if let Some(ex) = excludes {
+                let s = entry.path().to_str().unwrap_or("");
+                for exclude in ex {
+                    if s.contains(exclude) {
+                        // Hit an exclude dir, stop iterating.
+                        it.skip_current_dir();
+                        continue 'walkdir;
                     }
-                    true
-                } else {
-                    true
                 }
-            })
-            .filter_map(Result::ok)
-            .filter(|e| e.file_type().is_dir() && e.file_name() == ".git")
-        {
-            trace!("Entry: {:?}", &entry);
-            let mut repo_path = entry.into_path();
-            repo_path.pop();
-            repo_paths.push(repo_path);
+            }
+
+            // Add anything that has a .git dir inside it.
+            if entry.file_type().is_dir() && entry.path().join(".git").is_dir() {
+                // Found matching entry, add it.
+                trace!("Entry: {entry:?}");
+                repo_paths.push(entry.path().to_path_buf());
+
+                // Stop iterating, we don't want git repos inside other git repos.
+                it.skip_current_dir();
+            }
         }
     }
-    debug!("Found repo paths: {:?}", repo_paths);
+    debug!("Found repo paths: {repo_paths:?}");
     repo_paths
 }
 
-fn parse_git_config(path: &Path, prune: bool, remote_order: &[String]) -> Result<GitConfig> {
+fn parse_git_config(
+    path: &Path,
+    prune: bool,
+    remote_order: &[String],
+    home_dir: &str,
+) -> Result<GitConfig> {
     let repo = Repository::open(&path)?;
 
     let mut sorted_remote_names = Vec::new();
@@ -162,13 +190,21 @@ fn parse_git_config(path: &Path, prune: bool, remote_order: &[String]) -> Result
         )?);
     }
 
+    // Replace home directory in the path with ~.
+    let replaced_path = path.to_str().ok_or_else(|| E::InvalidUTF8Path {
+        path: path.to_path_buf(),
+    })?;
+    let replaced_path = replaced_path
+        .strip_prefix(home_dir)
+        .map_or_else(|| replaced_path.to_owned(), |p| format!("~{p}"));
+
     let config = GitConfig {
-        path: path.to_string_lossy().to_string(),
+        path: replaced_path,
         branch: None,
         remotes,
         prune,
     };
-    trace!("Parsed GitConfig: {:?}", &config);
+    trace!("Parsed GitConfig: {config:?}");
     Ok(config)
 }
 
@@ -179,6 +215,10 @@ pub enum GenerateGitError {
     InvalidUtf8,
     /// Invalid remote '{name}'.
     InvalidRemote { name: String },
+    /// Unable to calculate user's home directory.
+    MissingHomeDir,
     /// Unexpected None in option.
     UnexpectedNone,
+    /// Path contained invalid UTF-8 characters: {path:?}
+    InvalidUTF8Path { path: PathBuf },
 }
