@@ -15,7 +15,8 @@
 )]
 use color_eyre::eyre::Result;
 use log::trace;
-use opts::GenerateLib;
+use opts::{DefaultsSubcommand, GenerateLib};
+use tasks::{defaults, TasksAction, TasksDir};
 
 use crate::{
     config::UpConfig,
@@ -24,10 +25,13 @@ use crate::{
 
 mod config;
 mod env;
+pub mod errors;
+pub mod files;
 mod generate;
 pub mod opts;
 pub mod tasks;
-pub mod update;
+
+pub use files::get_up_dir;
 
 /// Run `up_rs` with provided [Opts][] struct.
 ///
@@ -41,20 +45,20 @@ pub mod update;
 ///
 /// [Opts]: crate::opts::Opts
 pub fn run(opts: Opts) -> Result<()> {
+    let up_dir = get_up_dir(opts.up_dir.as_ref());
     match opts.cmd {
-        // TODO(gib): Handle multiple link directories both as args and in config.
-        // TODO(gib): Add option to warn instead of failing if there are conflicts.
-        // TODO(gib): Check for conflicts before doing any linking.
         Some(SubCommand::Link(link_options)) => {
-            tasks::link::run(link_options)?;
+            tasks::link::run(link_options, &up_dir)?;
         }
         Some(SubCommand::Git(git_options)) => {
             tasks::git::update::update(&git_options.into())?;
         }
-        Some(SubCommand::Defaults {}) => {
-            // TODO(gib): implement defaults setting.
-            unimplemented!("Not yet implemented.");
-        }
+        Some(SubCommand::Defaults(defaults_options)) => match defaults_options.subcommand {
+            DefaultsSubcommand::Read(defaults_read_opts) => defaults::read(defaults_read_opts)?,
+            DefaultsSubcommand::Write(defaults_write_opts) => {
+                defaults::write(defaults_write_opts, &up_dir)?;
+            }
+        },
         Some(SubCommand::Self_(cmd_opts)) => {
             tasks::update_self::run(&cmd_opts)?;
         }
@@ -63,27 +67,29 @@ pub fn run(opts: Opts) -> Result<()> {
                 generate::git::run_single(git_opts)?;
             }
             Some(GenerateLib::Defaults(ref defaults_opts)) => {
-                trace!("Options: {:?}", defaults_opts);
+                trace!("Options: {defaults_opts:?}");
                 // TODO(gib): implement defaults generation.
-                unimplemented!("Allow generating defaults toml.");
+                unimplemented!("Allow generating defaults yaml.");
             }
             None => {
                 let config = UpConfig::from(opts)?;
                 generate::run(&config)?;
             }
         },
-        Some(SubCommand::Run(ref _cmd_opts)) => {
-            // TODO(gib): Store and fetch config in config module.
-            let config = UpConfig::from(opts)?;
-            update::update(&config)?;
-        }
         Some(SubCommand::Completions(ref cmd_opts)) => {
-            tasks::completions::run(cmd_opts)?;
+            tasks::completions::run(cmd_opts);
+        }
+        Some(SubCommand::List(ref _cmd_opts)) => {
+            let config = UpConfig::from(opts)?;
+            tasks::run(&config, TasksDir::Tasks, TasksAction::List)?;
+        }
+        Some(SubCommand::Run(ref _cmd_opts)) => {
+            let config = UpConfig::from(opts)?;
+            tasks::run(&config, TasksDir::Tasks, TasksAction::Run)?;
         }
         None => {
-            // TODO(gib): Store and fetch config in config module.
             let config = UpConfig::from(opts)?;
-            update::update(&config)?;
+            tasks::run(&config, TasksDir::Tasks, TasksAction::Run)?;
         }
     }
     Ok(())
