@@ -6,8 +6,6 @@
 //! being useful for maintaining large CI fleets, it is designed for a developer
 //! to use to manage the machines they regularly use.
 
-// TODO(gib): Good rust coverage checker (tarpaulin?)
-
 // #![feature(external_doc)]
 // #![doc(include = "../README.md")]
 
@@ -21,34 +19,37 @@ use std::{
     io,
     os::unix,
     path::PathBuf,
-    time::Instant,
+    time::{Duration, Instant},
 };
 
 use chrono::Utc;
 use color_eyre::eyre::{bail, Result};
 use displaydoc::Display;
-use log::{info, trace};
+use log::{debug, info, trace};
 use slog::{o, Drain, Duplicate, FnValue, LevelFilter, Logger};
 use thiserror::Error;
 use up_rs::opts::Color;
 
+/// Env vars to avoid printing when we log the current environment.
+const IGNORED_ENV_VARS: [&str; 1] = [
+    // Ignored because it's over 9,000 chars long, and irrelevant for up command debugging.
+    "LS_COLORS",
+];
+
+// TODO(gib): Return correct exit codes using https://docs.rs/exitcode/1.1.2/exitcode/.
 fn main() -> Result<()> {
     // Get starting time.
     let now = Instant::now();
 
-    // Default to showing backtraces.
-    if std::env::var("RUST_LIB_BACKTRACE").is_err() {
-        std::env::set_var("RUST_LIB_BACKTRACE", "1");
-    }
     color_eyre::install()?;
 
-    let args = up_rs::opts::parse();
+    let opts = up_rs::opts::parse();
 
     // TODO(gib): Don't need dates in stderr as we have them in file logger.
     // Create stderr logger.
     let stderr_decorator_builder = slog_term::TermDecorator::new().stderr();
 
-    let stderr_decorator = match &args.color {
+    let stderr_decorator = match &opts.color {
         Color::Auto => stderr_decorator_builder,
         Color::Always => stderr_decorator_builder.force_color(),
         Color::Never => stderr_decorator_builder.force_plain(),
@@ -60,30 +61,18 @@ fn main() -> Result<()> {
         .fuse();
     let stderr_async_drain = slog_async::Async::new(stderr_drain).build().fuse();
 
-    // TODO(gib): Parse input &args.log_level and set filter level.
-    // TODO(gib): Update --help text to note how you set the log level.
-    let stderr_level_filter = LevelFilter::new(stderr_async_drain, args.log_level);
+    let stderr_level_filter = LevelFilter::new(stderr_async_drain, opts.log_level);
 
-    let should_log_to_file = match &args.log_dir {
-        Some(p) if p == &PathBuf::new() => false,
-        Some(_) | None => true,
-    };
+    let LogPaths {
+        log_path,
+        log_path_link,
+        log_file,
+    } = get_log_path_file(opts.up_dir.as_ref())
+        .map_err(|e| MainError::LogFileSetupFailed { source: e })?;
+    // Create file logger.
+    let file_decorator = slog_term::PlainSyncDecorator::new(log_file);
+    let file_drain = slog_term::FullFormat::new(file_decorator).build().fuse();
 
-    let mut file_drain = None;
-    let mut log_paths_opt = None;
-    if should_log_to_file {
-        let LogPaths {
-            log_path,
-            log_path_link,
-            log_file,
-        } = get_log_path_file(args.log_dir.as_ref())
-            .map_err(|e| MainError::LogFileSetupFailed { source: e })?;
-        log_paths_opt = Some((log_path, log_path_link));
-
-        // Create file logger.
-        let file_decorator = slog_term::PlainSyncDecorator::new(log_file);
-        file_drain = Some(slog_term::FullFormat::new(file_decorator).build().fuse());
-    }
     let log_kv_pairs = o!("place" =>
        FnValue(move |info| {
            format!("{}:{} {}",
@@ -94,16 +83,11 @@ fn main() -> Result<()> {
        })
     );
 
-    #[allow(clippy::option_if_let_else)]
-    let root_logger = if let Some(file_drain) = file_drain {
-        let file_level_filter = LevelFilter::new(file_drain, args.file_log_level);
-        Logger::root(
-            Duplicate::new(stderr_level_filter, file_level_filter).fuse(),
-            log_kv_pairs,
-        )
-    } else {
-        Logger::root(stderr_level_filter.fuse(), log_kv_pairs)
-    };
+    let file_level_filter = LevelFilter::new(file_drain, opts.file_log_level);
+    let root_logger = Logger::root(
+        Duplicate::new(stderr_level_filter, file_level_filter).fuse(),
+        log_kv_pairs,
+    );
 
     // slog_stdlog uses the logger from slog_scope, so set a logger there
     // In the future probably want to use proper scoped loggers.
@@ -113,20 +97,25 @@ fn main() -> Result<()> {
     slog_stdlog::init()?;
 
     trace!("Starting up.");
-    if let Some((log_path, log_path_link)) = log_paths_opt {
-        info!(
-            "Writing full logs to {} (symlink to '{}')",
-            &log_path_link.display(),
-            &log_path.display()
-        );
+    debug!("Writing full logs to {log_path_link:?} (symlink to '{log_path:?}')",);
+
+    trace!("Received args: {opts:#?}");
+    trace!(
+        "Current env: {:?}",
+        env::vars()
+            .filter(|(k, _v)| !IGNORED_ENV_VARS.contains(&k.as_str()))
+            .collect::<Vec<_>>()
+    );
+
+    up_rs::run(opts)?;
+
+    // No need to log the time we took to run by default unless it actually took some time.
+    let now_elapsed = now.elapsed();
+    if now_elapsed > Duration::from_secs(10) {
+        info!("Up-rs ran successfully in {now_elapsed:?}");
+    } else {
+        debug!("Up-rs ran successfully in {now_elapsed:?}");
     }
-
-    trace!("Received args: {:#?}", args);
-    trace!("Current env: {:?}", env::vars().collect::<Vec<_>>());
-
-    up_rs::run(args)?;
-
-    info!("Up-rs ran successfully in {:?}", now.elapsed());
     trace!("Finished up.");
     Ok(())
 }
@@ -144,11 +133,9 @@ struct LogPaths {
 
 /// Create log file, and a symlink to it that can be used to find the latest
 /// one.
-fn get_log_path_file(log_dir: Option<&PathBuf>) -> Result<LogPaths> {
-    let log_dir = log_dir.map_or_else(
-        || env::temp_dir().join("up-rs/logs"),
-        std::clone::Clone::clone,
-    );
+fn get_log_path_file(up_dir_opt: Option<&PathBuf>) -> Result<LogPaths> {
+    let mut log_dir = up_rs::get_up_dir(up_dir_opt);
+    log_dir.push("logs");
     fs::create_dir_all(&log_dir).map_err(|e| MainError::CreateDirError {
         path: log_dir.clone(),
         source: e,
