@@ -1,16 +1,19 @@
 use std::convert::From;
 
-use clap::Clap;
-use color_eyre::eyre::{eyre, Context, Result};
+use clap::Parser;
+use color_eyre::eyre::Result;
 use displaydoc::Display;
 use git2::Remote;
 use log::error;
-use rayon::prelude::*;
+use rayon::{iter::Either, prelude::*};
 use serde_derive::{Deserialize, Serialize};
 use thiserror::Error;
 
 use self::GitTaskError as E;
-use crate::{opts::GitOptions, tasks::ResolveEnv};
+use crate::{
+    opts::GitOptions,
+    tasks::{task::TaskStatus, ResolveEnv, TaskError},
+};
 
 pub mod branch;
 pub mod checkout;
@@ -33,6 +36,7 @@ pub struct GitConfig {
     /// Branch to checkout when cloning/updating. Defaults to the current branch
     /// when updating, or the default branch of the first remote for
     /// cloning.
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub branch: Option<String>,
     /// Prune local branches whose changes have already been merged upstream.
     #[serde(default = "prune_default")]
@@ -44,25 +48,27 @@ const fn prune_default() -> bool {
     false
 }
 
-// TODO(gib): Pass by reference instead.
-#[allow(clippy::needless_pass_by_value)]
-pub(crate) fn run(configs: Vec<GitConfig>) -> Result<()> {
-    // TODO(gib): run them in parallel.
-    // TODO(gib): continue even if one errors.
-    let errors: Vec<_> = configs
+pub(crate) fn run(configs: &[GitConfig]) -> Result<TaskStatus> {
+    let (statuses, errors): (Vec<_>, Vec<_>) = configs
         .par_iter()
-        .map(|c| update::update(c))
-        .filter_map(Result::err)
-        .collect();
+        .map(update::update)
+        .partition_map(|x| match x {
+            Ok(status) => Either::Left(status),
+            Err(e) => Either::Right(e),
+        });
+
     if errors.is_empty() {
-        Ok(())
+        if statuses.iter().all(|s| matches!(s, TaskStatus::Skipped)) {
+            Ok(TaskStatus::Skipped)
+        } else {
+            Ok(TaskStatus::Passed)
+        }
     } else {
         for error in &errors {
-            error!("{:?}", error);
+            error!("{error:?}");
         }
-        let mut errors_iter = errors.into_iter();
-        Err(errors_iter.next().ok_or(E::UnexpectedNone)?)
-            .with_context(|| eyre!("{:?}", errors_iter.collect::<Vec<_>>()))
+        let first_error = errors.into_iter().next().ok_or(E::UnexpectedNone)?;
+        Err(first_error)
     }
 }
 
@@ -82,9 +88,9 @@ impl From<GitOptions> for GitConfig {
 }
 
 impl ResolveEnv for Vec<GitConfig> {
-    fn resolve_env<F>(&mut self, env_fn: F) -> Result<()>
+    fn resolve_env<F>(&mut self, env_fn: F) -> Result<(), TaskError>
     where
-        F: Fn(&str) -> Result<String>,
+        F: Fn(&str) -> Result<String, TaskError>,
     {
         for config in self.iter_mut() {
             if let Some(branch) = config.branch.as_ref() {
@@ -105,13 +111,14 @@ impl ResolveEnv for Vec<GitConfig> {
     }
 }
 
-#[derive(Debug, Default, Clap, Serialize, Deserialize)]
+#[derive(Debug, Default, Parser, Serialize, Deserialize)]
 pub struct GitRemote {
     /// Name of the remote to set in git.
     pub name: String,
     /// URL to fetch from. Also used for pushing if `push_url` unset.
     pub fetch_url: String,
     /// URL to push to, defaults to fetch URL.
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub push_url: Option<String>,
 }
 
