@@ -7,15 +7,17 @@ use log::debug;
 use super::status::ensure_repo_clean;
 use crate::tasks::git::{checkout::set_and_checkout_head, errors::GitError as E};
 
-pub(super) fn do_merge<'a>(
+/// Fast-forward merge, returns "Skipped" if nothing was updated.
+/// Returns whether we did any work (`false` means skipped).
+pub(super) fn do_ff_merge<'a>(
     repo: &'a Repository,
     branch_name: &str,
     fetch_commit: &git2::AnnotatedCommit<'a>,
-) -> Result<()> {
+) -> Result<bool> {
     // Do merge analysis
     let analysis = repo.merge_analysis(&[fetch_commit])?;
 
-    debug!("Merge analysis: {:?}", &analysis);
+    debug!("Merge analysis: {analysis:?}");
 
     // Do the merge
     if analysis.0.is_fast_forward() {
@@ -31,19 +33,20 @@ pub(super) fn do_merge<'a>(
                 branch_name,
                 fetch_commit.id(),
                 true,
-                &format!("Setting {} to {}", branch_name, fetch_commit.id()),
+                &format!("Setting {branch_name} to {}", fetch_commit.id()),
             )?;
             set_and_checkout_head(repo, branch_name, false)?;
         }
+        Ok(true)
     } else if analysis.0.is_up_to_date() {
         debug!("Skipping fast-forward merge as already up-to-date.");
+        Ok(false)
     } else {
         bail!(E::CannotFastForwardMerge {
             analysis: analysis.0,
             preference: analysis.1
         });
     }
-    Ok(())
 }
 
 fn fast_forward(repo: &Repository, lb: &mut Reference, rc: &git2::AnnotatedCommit) -> Result<()> {
@@ -51,8 +54,8 @@ fn fast_forward(repo: &Repository, lb: &mut Reference, rc: &git2::AnnotatedCommi
         Some(s) => s.to_string(),
         None => String::from_utf8_lossy(lb.name_bytes()).to_string(),
     };
-    let msg = format!("Fast-Forward: Setting {} to id: {}", name, rc.id());
-    debug!("{}", msg);
+    let msg = format!("Fast-Forward: Setting {name} to id: {}", rc.id());
+    debug!("{msg}");
     ensure_repo_clean(repo)?;
     lb.set_target(rc.id(), &msg)?;
     // Force checkout as we already changed what the HEAD branch points to, and we
