@@ -11,46 +11,46 @@ use log::{debug, info, trace, warn};
 use thiserror::Error;
 use walkdir::{DirEntry, WalkDir};
 
-use crate::{opts::LinkOptions, tasks::ResolveEnv};
+use crate::{
+    files,
+    opts::LinkOptions,
+    tasks::{task::TaskStatus, ResolveEnv, TaskError},
+};
 
 impl ResolveEnv for LinkOptions {
-    fn resolve_env<F>(&mut self, env_fn: F) -> Result<()>
+    fn resolve_env<F>(&mut self, env_fn: F) -> Result<(), TaskError>
     where
-        F: Fn(&str) -> Result<String>,
+        F: Fn(&str) -> Result<String, TaskError>,
     {
         self.from_dir = env_fn(&self.from_dir)?;
         self.to_dir = env_fn(&self.to_dir)?;
-        self.backup_dir = env_fn(&self.backup_dir)?;
         Ok(())
     }
 }
 
 /// Symlink everything from `to_dir` (default: ~/code/dotfiles/) into `from_dir`
 /// (default: ~). Anything that would be overwritten is copied into `backup_dir`
-/// (default: ~/backup/).
+/// (default: `up_dir/backup/link/`).
 ///
 /// Basically you put your dotfiles in ~/code/dotfiles/, in the same structure
 /// they were in relative to ~. Then if you want to edit your .bashrc (for
 /// example) you just edit ~/.bashrc, and as it's a symlink it'll actually edit
 /// ~/code/dotfiles/.bashrc. Then you can add and commit that change in ~/code/
 /// dotfiles.
-pub(crate) fn run(config: LinkOptions) -> Result<()> {
+pub(crate) fn run(config: LinkOptions, up_dir: &Path) -> Result<TaskStatus> {
     let now: DateTime<Utc> = Utc::now();
-    debug!("UTC time is: {}", now);
+    debug!("UTC time is: {now}");
 
     let from_dir = PathBuf::from(config.from_dir);
     let to_dir = PathBuf::from(config.to_dir);
-    let backup_dir = PathBuf::from(config.backup_dir);
+    let backup_dir = up_dir.join("backup/link");
 
     let from_dir = resolve_directory(from_dir, "From")?;
     let to_dir = resolve_directory(to_dir, "To")?;
 
     // Create the backup dir if it doesn't exist.
     if !backup_dir.exists() {
-        debug!(
-            "Backup dir '{}' doesn't exist, creating it.",
-            backup_dir.display()
-        );
+        debug!("Backup dir '{backup_dir:?}' doesn't exist, creating it.",);
         fs::create_dir_all(&backup_dir).map_err(|e| LinkError::CreateDirError {
             path: backup_dir.clone(),
             source: e,
@@ -58,10 +58,7 @@ pub(crate) fn run(config: LinkOptions) -> Result<()> {
     }
     let backup_dir = resolve_directory(backup_dir, "Backup")?;
 
-    debug!(
-        "Linking from {:?} to {:?} (backup dir {:?}).",
-        from_dir, to_dir, backup_dir
-    );
+    debug!("Linking from {from_dir:?} to {to_dir:?} (backup dir {backup_dir:?}).",);
     debug!(
         "to_dir contents: {:?}",
         fs::read_dir(&to_dir)?
@@ -71,6 +68,7 @@ pub(crate) fn run(config: LinkOptions) -> Result<()> {
             .collect::<Result<Vec<_>>>()
     );
 
+    let mut work_done = false;
     // For each non-directory file in from_dir.
     for from_path in WalkDir::new(&from_dir)
         .min_depth(1)
@@ -80,15 +78,14 @@ pub(crate) fn run(config: LinkOptions) -> Result<()> {
     {
         let rel_path = from_path.path().strip_prefix(&from_dir)?;
         create_parent_dir(&to_dir, rel_path, &backup_dir)?;
-        link_path(&from_path, &to_dir, rel_path, &backup_dir)?;
+        if link_path(&from_path, &to_dir, rel_path, &backup_dir)? {
+            work_done = true;
+        }
     }
 
     // Remove backup dir if not empty.
     if let Err(err) = fs::remove_dir(&backup_dir) {
-        warn!(
-            "Backup dir {:?} non-empty, check contents: {:?}",
-            backup_dir, err
-        );
+        warn!("Backup dir {backup_dir:?} non-empty, check contents: {err:?}",);
     }
 
     debug!(
@@ -107,7 +104,11 @@ pub(crate) fn run(config: LinkOptions) -> Result<()> {
         );
     }
 
-    Ok(())
+    if work_done {
+        Ok(TaskStatus::Passed)
+    } else {
+        Ok(TaskStatus::Skipped)
+    }
 }
 
 /// Ensure dir exists, and resolve symlinks to find it's canonical path.
@@ -136,7 +137,7 @@ fn create_parent_dir(to_dir: &Path, rel_path: &Path, backup_dir: &Path) -> Resul
     fs::create_dir_all(to_path_parent).or_else(|_err| {
         info!("Failed to create parent dir, walking up the tree to see if there's a file that needs to become a directory.");
         for path in rel_path.ancestors().skip(1).filter(|p| p != &Path::new("")) {
-            debug!("Checking path {:?}", path);
+            debug!("Checking path {path:?}");
             let abs_path = to_dir.join(path);
             // The path is a file/dir/symlink, or a broken symlink.
             if abs_path.exists() || abs_path.symlink_metadata().is_ok() {
@@ -145,25 +146,23 @@ fn create_parent_dir(to_dir: &Path, rel_path: &Path, backup_dir: &Path) -> Resul
                         abs_path);
                 warn!(
                     "File will be overwritten by parent directory of link.\n  \
-                     File: {:?}\n  Link: {:?}",
-                    &abs_path, &to_path
+                     File: {abs_path:?}\n  Link: {to_path:?}",
                 );
                 if abs_path.is_file() {
                     if let Some(parent_path) = &path.parent() {
-                        info!("Path: {:?}, parent: {:?}", path, parent_path);
+                        info!("Path: {path:?}, parent: {parent_path:?}");
                         if parent_path != &Path::new("") {
                             let path = backup_dir.join(parent_path);
                             fs::create_dir_all(&path).map_err(|e| LinkError::CreateDirError{path, source: e})?;
                         }
                         let backup_path = backup_dir.join(path);
                         info!(
-                            "Moving file to backup: {:?} -> {:?}",
-                            &abs_path, &backup_path
+                            "Moving file to backup: {abs_path:?} -> {backup_path:?}",
                         );
                         fs::rename(&abs_path, backup_path)?;
                     }
                 } else {
-                    info!("Removing symlink: {:?}", abs_path);
+                    info!("Removing symlink: {abs_path:?}");
                     fs::remove_file(abs_path)?;
                 }
             }
@@ -183,13 +182,14 @@ fn get_parent_path(path: &Path) -> Result<&Path> {
 /// Create a symlink from `from_path` -> `to_path`.
 /// `rel_path` is the relative path within `from_dir`.
 /// Moves any existing files that would be overwritten into `backup_dir`.
+/// Returns a boolean indicating whether any symlinks were created.
 #[allow(clippy::filetype_is_file)]
 fn link_path(
     from_path: &DirEntry,
     to_dir: &Path,
     rel_path: &Path,
     backup_dir: &Path,
-) -> Result<()> {
+) -> Result<bool> {
     let to_path = to_dir.join(rel_path);
     if to_path.exists() {
         let to_path_file_type = to_path.symlink_metadata()?.file_type();
@@ -198,15 +198,12 @@ fn link_path(
                 Ok(existing_link) => {
                     if existing_link == from_path.path() {
                         debug!(
-                            "Link at {:?} already points to {:?}, skipping.",
-                            to_path, existing_link
+                            "Link at {to_path:?} already points to {existing_link:?}, skipping.",
                         );
-                        return Ok(());
+                        return Ok(false);
                     }
                     warn!(
-                        "Link at {:?} points to {:?}, changing to {:?}.",
-                        to_path,
-                        existing_link,
+                        "Link at {to_path:?} points to {existing_link:?}, changing to {:?}.",
                         from_path.path()
                     );
                     fs::remove_file(&to_path).map_err(|e| LinkError::DeleteError {
@@ -215,13 +212,12 @@ fn link_path(
                     })?;
                 }
                 Err(e) => {
-                    bail!("read_link returned error {:?} for {:?}", e, to_path);
+                    bail!("read_link returned error {e:?} for {to_path:?}");
                 }
             }
         } else if to_path_file_type.is_dir() {
             warn!(
-                "Expected file or link at {:?}, found directory, moving to {:?}",
-                to_path, backup_dir
+                "Expected file or link at {to_path:?}, found directory, moving to {backup_dir:?}",
             );
             let backup_path = backup_dir.join(rel_path);
             fs::create_dir_all(&backup_path).map_err(|e| LinkError::CreateDirError {
@@ -234,7 +230,7 @@ fn link_path(
                 source: e,
             })?;
         } else if to_path_file_type.is_file() {
-            warn!("Existing file at {:?}, moving to {:?}", to_path, backup_dir);
+            warn!("Existing file at {to_path:?}, moving to {backup_dir:?}");
             let backup_path = backup_dir.join(rel_path);
             let backup_parent_path = get_parent_path(&backup_path)?;
             fs::create_dir_all(backup_parent_path).map_err(|e| LinkError::CreateDirError {
@@ -250,30 +246,22 @@ fn link_path(
             bail!("This should be unreachable.")
         }
     } else if to_path.symlink_metadata().is_ok() {
-        warn!(
-            "Removing existing broken link.\n  Path: {:?}\n  Dest: {:?}",
-            &to_path,
-            &to_path.read_link().map_err(|e| LinkError::IoError {
-                path: to_path.clone(),
-                source: e
-            })?
-        );
-        fs::remove_file(&to_path).map_err(|e| LinkError::DeleteError {
-            path: to_path.clone(),
-            source: e,
-        })?;
+        files::remove_broken_symlink(&to_path)?;
     } else {
-        trace!("File '{:?}' doesn't exist.", to_path);
+        trace!("File '{to_path:?}' doesn't exist.");
     }
-    info!("Linking:\n  From: {:?}\n  To: {:?}", from_path, to_path);
-    unix::fs::symlink(from_path.path(), &to_path).map_err(|e| {
-        LinkError::SymlinkError {
-            from_path: from_path.path().to_path_buf(),
-            to_path: to_path.clone(),
-            source: e,
-        }
-        .into()
-    })
+    info!("Linking:\n  From: {from_path:?}\n  To: {to_path:?}");
+    unix::fs::symlink(from_path.path(), &to_path)
+        // If we got here, we did work, so return true.
+        .map(|()| true)
+        .map_err(|e| {
+            LinkError::SymlinkError {
+                from_path: from_path.path().to_path_buf(),
+                to_path: to_path.clone(),
+                source: e,
+            }
+            .into()
+        })
 }
 
 #[derive(Error, Debug, Display)]
