@@ -15,7 +15,10 @@ use serde_derive::Deserialize;
 use thiserror::Error;
 
 use self::UpdateSelfError as E;
-use crate::{opts::UpdateSelfOptions, tasks::ResolveEnv};
+use crate::{
+    opts::UpdateSelfOptions,
+    tasks::{task::TaskStatus, ResolveEnv},
+};
 
 #[derive(Debug, Deserialize)]
 struct GitHubReleaseJsonResponse {
@@ -30,50 +33,46 @@ impl ResolveEnv for UpdateSelfOptions {}
 
 /// Downloads the latest version of the binary from the specified URL and
 /// replaces the current executable path with it.
-pub(crate) fn run(opts: &UpdateSelfOptions) -> Result<()> {
+pub(crate) fn run(opts: &UpdateSelfOptions) -> Result<TaskStatus> {
     let up_path = env::current_exe()?.canonicalize().unwrap();
 
     // If the current binary's location is where it was originally compiled, assume it is a dev
     // build, and thus skip the update.
     if !opts.always_update && up_path.starts_with(env!("CARGO_MANIFEST_DIR")) {
-        info!(
-            "Skipping up-rs update, current version '{}' is a dev build.",
-            &up_path.display(),
-        );
-        return Ok(());
+        debug!("Skipping up-rs update, current version '{up_path:?}' is a dev build.",);
+        return Ok(TaskStatus::Skipped);
     }
 
     let client = reqwest::blocking::Client::builder()
         .user_agent(APP_USER_AGENT)
         .build()?;
 
+    trace!("Self update opts: {opts:?}");
     if opts.url == crate::opts::SELF_UPDATE_URL {
         let latest_github_release = client
             .get(crate::opts::LATEST_RELEASE_URL)
             .send()?
             .error_for_status()?
             .json::<GitHubReleaseJsonResponse>()?;
-        trace!("latest_github_release: {:?}", latest_github_release,);
+        trace!("latest_github_release: {latest_github_release:?}");
         let latest_github_release = latest_github_release.tag_name;
-        if CURRENT_VERSION == latest_github_release {
-            info!(
-                "Skipping up-rs update, current version '{}' is latest GitHub version '{:?}'",
-                CURRENT_VERSION, &latest_github_release,
+        if semver::Version::parse(&latest_github_release)?
+            <= semver::Version::parse(CURRENT_VERSION)?
+        {
+            debug!(
+                "Skipping up-rs update, current version '{CURRENT_VERSION}' is not older than latest GitHub version '{latest_github_release}'",
             );
-            return Ok(());
+            return Ok(TaskStatus::Skipped);
         }
+        trace!("Updating up-rs from '{CURRENT_VERSION}' to '{latest_github_release}'",);
     }
 
     let temp_dir = env::temp_dir();
     let temp_path = &temp_dir.join(format!("up_rs-{}", Utc::now().to_rfc3339()));
 
-    debug!(
-        "Downloading url {} to path {}",
-        &opts.url,
-        up_path.display()
-    );
+    trace!("Downloading url {url} to path {up_path:?}", url = &opts.url,);
 
-    debug!("Using temporary path: {}", temp_path.display());
+    trace!("Using temporary path: {temp_path:?}");
     let mut response = reqwest::blocking::get(&opts.url)?.error_for_status()?;
 
     fs::create_dir_all(&temp_dir).with_context(|| E::CreateDir { path: temp_dir })?;
@@ -93,21 +92,18 @@ pub(crate) fn run(opts: &UpdateSelfOptions) -> Result<()> {
         .trim()
         .trim_start_matches(concat!(env!("CARGO_PKG_NAME"), " "));
     if semver::Version::parse(new_version)? > semver::Version::parse(CURRENT_VERSION)? {
-        info!(
-            "Updating up-rs from '{}' to '{}'",
-            CURRENT_VERSION, &new_version,
-        );
+        info!("Updating up-rs from '{CURRENT_VERSION}' to '{new_version}'",);
         fs::rename(&temp_path, &up_path).with_context(|| E::Rename {
             from: temp_path.clone(),
             to: up_path.clone(),
         })?;
+        Ok(TaskStatus::Passed)
     } else {
-        info!(
-            "Skipping up-rs update, current version '{}' and new version '{}'",
-            CURRENT_VERSION, &new_version,
+        debug!(
+            "Skipping up-rs update, current version '{CURRENT_VERSION}' and new version '{new_version}'",
         );
+        Ok(TaskStatus::Skipped)
     }
-    Ok(())
 }
 
 #[derive(Error, Debug, Display)]
